clear; clc; close all;

% ----- Light ----- %
wvl = 635e-9; % wavelength
k = 2*pi/wvl; % wavenumber
P = 2.5e-3;   % output power
% ----------------- %

% ----- MMF ----- %
NA = 0.22;   % Numerical Aperture
Ds = 105e-6; % core diameter
% --------------- %

% ----- 4f ----- %
f1 = 100e-3; % f1 focal length
f2 = 500e-3; % f2 focal length
M = -f2/f1;  % magnification
% -------------- %

% ----- Grids ----- %
N = 512;
dsrc = 4*Ds/N;
[xsrc,ysrc] = meshgrid((-N/2:N/2-1)*dsrc);
[fxsrc,fysrc] = meshgrid((-N/2:N/2-1)*1/(N*dsrc));
[~,rhosrc] = cart2pol(xsrc,ysrc);
[xf1,yf1] = meshgrid((-N/2:N/2-1)*wvl*f1/(N*dsrc));
[xf2,yf2] = meshgrid((-N/2:N/2-1)*wvl*f2/(N*mean(diff(xf1(1,:)))));
[~,rhof2] = cart2pol(xf2,yf2);
% ----------------- %

% ----- MMF Partially Coherent Source ----- %
alpha = 1/2;
mu_src = jinc(1/pi*k*NA*rhosrc*alpha);                  % MMF SDoC
mu_src_tilde = fftshift(fft2(fftshift(mu_src)))*dsrc^2;
S_src = circ(xsrc,ysrc,Ds);                             % MMF SD            
S_src_tilde = fftshift(fft2(fftshift(S_src)))*dsrc^2;
% ----------------------------------------- %

% ----- Desired Output SDoC ----- %
Celtic_cross = double(rgb2gray(imread('Celtic_cross.jpg')));
Celtic_cross = imresize(Celtic_cross,0.4);
if rem(size(Celtic_cross,1),2) ~= 0
    Celtic_cross(end,:) = '';
end
if rem(size(Celtic_cross,2),2) ~= 0
    Celtic_cross(:,end) = '';
end
Celtic_cross = padarray(Celtic_cross,[N/2-size(Celtic_cross,1)/2 N/2-size(Celtic_cross,2)/2]);
T = sqrt(Celtic_cross);
T = T/max(max(T));
T = rot90(T,2);
% ------------------------------- %

% ----- Simulation ----- %
numscreens = 2000;
S_sim_out = zeros(N);
S_sim_fz = zeros(N);
for ii = 1:numscreens
    Usrc = Generate_Complex_Screen_Field_Instance(S_src,mu_src_tilde,dsrc,1);
    [xf,yf,Uf] = lens_in_front_ft(Usrc,wvl,dsrc,f1,f1);
    [xout,yout,Uout] = lens_in_front_ft(T.*Uf,wvl,mean(diff(xf(1,:))),f2,f2);
    S_sim_out = S_sim_out + abs(Uout).^2;
    [Ufz,xfz,yfz] = fraunhofer_prop(Uout,wvl,mean(diff(xout(1,:))),1);
    S_sim_fz = S_sim_fz + abs(Ufz).^2;
end
S_sim_out = S_sim_out/numscreens;     % simulated output SD
S_sim_fz = S_sim_fz/numscreens;       % simulated far-zone SD
% ---------------------- %

% ----- Theoretical Output Partially Coherent Field ----- %
mu_filt_0 = abs(trapz(fysrc(:,1),trapz(fxsrc(1,:),abs(T).^2.*mu_src_tilde,2),1));
S_thy = 1/M^2*circ(xout/M,yout/M,Ds)*mu_filt_0;                             % theoretical output SD
% ------------------------------------------------------- %

% ----- Theoretical Far-Zone SD ----- %
S_thy_fz = M^2/(wvl^2)*S_src_tilde(N/2+1,N/2+1)*mu_src_tilde(N/2+1,N/2+1) ...
    *abs(rot90(T,2)).^2;
% ----------------------------------- %

figure;
set(gcf,'paperpositionmode','auto');
set(gcf,'position',[731 343 720 384]);

subplot(231);
imagesc(xout(1,:)/(Ds/2),yout(:,1)/(Ds/2),S_thy);
axis image; h = colorbar;
xlabel('$2x/D_c$','interpreter','latex','fontsize',10);
ylabel('$2y/D_c$','interpreter','latex','fontsize',10);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
axis([-1 1 -1 1]*10);
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-8.891,-7.583,'$\mbox{(a)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.057 0.605 0.204 0.341]);
set(h,'position',[0.258 0.605 0.027 0.341]);

subplot(232);
imagesc(xout(1,:)/(Ds/2),yout(:,1)/(Ds/2),S_sim_out);
axis image; h = colorbar;
xlabel('$2x/D_c$','interpreter','latex','fontsize',10);
ylabel('$2y/D_c$','interpreter','latex','fontsize',10);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
axis([-1 1 -1 1]*10);
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-8.891,-7.583,'$\mbox{(b)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.396 0.605 0.204 0.341]);
set(h,'position',[0.597 0.605 0.027 0.341]);

subplot(233);
imagesc(xout(1,:)/(Ds/2),yout(:,1)/(Ds/2),abs(S_sim_out - S_thy));
axis image; h = colorbar;
xlabel('$2x/D_c$','interpreter','latex','fontsize',10);
ylabel('$2y/D_c$','interpreter','latex','fontsize',10);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
axis([-1 1 -1 1]*10);
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-8.891,-7.583,'$\mbox{(c)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.72 0.605 0.204 0.341]);
set(h,'position',[0.921 0.605 0.027 0.341]);

subplot(234);
imagesc(xf(1,:)*2*1000,yf(:,1)*2*1000,S_thy_fz);
axis image; h = colorbar;
axis([-1 1 -1 1]*20); 
xlabel('$x/z \times 10^3$','interpreter','latex','fontsize',10);
ylabel('$y/z \times 10^3$','interpreter','latex','fontsize',10);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-17.923,-15.342,'$\mbox{(d)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.057 0.104 0.204 0.341]);
set(h,'position',[0.258 0.104 0.027 0.341]);

subplot(235);
imagesc(xfz(1,:)*1000,yfz(:,1)*1000,S_sim_fz); 
axis image; h = colorbar; 
axis([-1 1 -1 1]*20);
xlabel('$x/z \times 10^3$','interpreter','latex','fontsize',10);
ylabel('$y/z \times 10^3$','interpreter','latex','fontsize',10);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-17.923,-15.342,'$\mbox{(e)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.396 0.104 0.204 0.341]);
set(h,'position',[0.597 0.104 0.027 0.341]);

subplot(236);
imagesc(xfz(1,:)*1000,yfz(:,1)*1000,abs(S_sim_fz - S_thy_fz)); 
axis image; h = colorbar; 
axis([-1 1 -1 1]*20); 
xlabel('$x/z \times 10^3$','interpreter','latex','fontsize',10);
ylabel('$y/z \times 10^3$','interpreter','latex','fontsize',10);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-17.923,-15.342,'$\mbox{(f)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.72 0.104 0.204 0.341]);
set(h,'position',[0.921 0.104 0.027 0.341]);