clear; clc; close all;

% ----- Light ----- %
wvl = 635e-9; % wavelength
k = 2*pi/wvl; % wavenumber
P = 2.5e-3;   % output power
% ----------------- %

% ----- MMF ----- %
NA = 0.22;   % Numerical Aperture
Ds = 105e-6; % core diameter
% --------------- %

% ----- 4f ----- %
f1 = 100e-3; % f1 focal length
f2 = 500e-3; % f2 focal length
M = -f2/f1;  % magnification
% -------------- %

% ----- Grids ----- %
N = 512;
dsrc = 4*Ds/N;
[xsrc,ysrc] = meshgrid((-N/2:N/2-1)*dsrc);
[fxsrc,fysrc] = meshgrid((-N/2:N/2-1)*1/(N*dsrc));
[~,rhosrc] = cart2pol(xsrc,ysrc);
[xf1,yf1] = meshgrid((-N/2:N/2-1)*wvl*f1/(N*dsrc));
[xf2,yf2] = meshgrid((-N/2:N/2-1)*wvl*f2/(N*mean(diff(xf1(1,:)))));
[~,rhof2] = cart2pol(xf2,yf2);
% ----------------- %

% ----- MMF Partially Coherent Source ----- %
alpha = 1/2;
mu_src = jinc(1/pi*k*NA*rhosrc*alpha);                  % MMF SDoC
mu_src_tilde = fftshift(fft2(fftshift(mu_src)))*dsrc^2;
S_src = circ(xsrc,ysrc,Ds);                             % MMF SD            
S_src_tilde = fftshift(fft2(fftshift(S_src)))*dsrc^2;
% ----------------------------------------- %

% ----- Desired Output SDoC ----- %
mu_out = (sinc(xf2/(2*Ds)).*sinc(yf2/(4*Ds))).^2;
psd_out = abs(fftshift(fft2(fftshift(mu_out)))*mean(diff(xf2(1,:)))^2);
T = sqrt(psd_out);
T = T/max(max(T));
T = rot90(T,2);
% ------------------------------- %

% ----- Simulation ----- %
numscreens = 2000;
S_sim = zeros(N);
Ucorr = zeros(N);
for ii = 1:numscreens
    Usrc = Generate_Complex_Screen_Field_Instance(S_src,mu_src_tilde,dsrc,1);
    [xf,yf,Uf] = lens_in_front_ft(Usrc,wvl,dsrc,f1,f1);
    [xout,yout,Uout] = lens_in_front_ft(T.*Uf,wvl,mean(diff(xf(1,:))),f2,f2);
    Ucorr = Ucorr + FFT_Correlation_2D(Uout,Uout,mean(diff(xout(1,:))));
    S_sim = S_sim + abs(Uout).^2;
end
mu_sim = abs(Ucorr)/Ucorr(N/2+1,N/2+1); % simulated output SDoC
S_sim = S_sim/numscreens;               % simulated output SD
% ---------------------- %

% ----- Theoretical Output Partially Coherent Field ----- %
mu_filt_0 = abs(trapz(fysrc(:,1),trapz(fxsrc(1,:),abs(T).^2.*mu_src_tilde,2),1));
S_thy = 1/M^2*circ(xout/M,yout/M,Ds)*mu_filt_0;                             % theoretical output SD
mu_thy = abs(mu_out);                                                       % theoretical output SDoC
% ------------------------------------------------------- %

figure;
set(gcf,'paperpositionmode','auto');
set(gcf,'position',[595 355 720 384]);

subplot(241);
imagesc(xout(1,:)/(Ds/2),yout(:,1)/(Ds/2),S_thy);
axis image; h = colorbar;
xlabel('$2x/D_c$','interpreter','latex','fontsize',10,'position',[0 13.643]);
ylabel('$2y/D_c$','interpreter','latex','fontsize',10,'position',[-13.322 0]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
axis([-1 1 -1 1]*10);
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-8.835,-7.2,'$\mbox{(a)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.056+0.015 0.646 0.135 0.341]);
set(h,'position',[0.197+0.015 0.69 0.02 0.253]);

subplot(242);
imagesc(xout(1,:)/(Ds/2),yout(:,1)/(Ds/2),S_sim);
axis image; h = colorbar;
xlabel('$2x/D_c$','interpreter','latex','fontsize',10,'position',[0 13.643]);
ylabel('$2y/D_c$','interpreter','latex','fontsize',10,'position',[-13.322 0]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
axis([-1 1 -1 1]*10);
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-8.835,-7.2,'$\mbox{(b)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.297+0.015 0.646 0.135 0.341]);
set(h,'position',[0.438+0.015 0.69 0.02 0.253]);

subplot(243);
imagesc(xout(1,:)/(Ds/2),yout(:,1)/(Ds/2),mu_thy);
axis image; h = colorbar;
axis([-1 1 -1 1]*20);
xlabel('$2x/D_c$','interpreter','latex','fontsize',10,'position',[0 27.211]);
ylabel('$2y/D_c$','interpreter','latex','fontsize',10,'position',[-26.576 0]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-17.44,-14.175,'$\mbox{(c)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.538+0.015 0.646 0.135 0.341]);
set(h,'position',[0.679+0.015 0.69 0.02 0.253]);

subplot(244);
imagesc(xout(1,:)/(Ds/2),yout(:,1)/(Ds/2),mu_sim);
axis image; h = colorbar;
axis([-1 1 -1 1]*20);
xlabel('$2x/D_c$','interpreter','latex','fontsize',10,'position',[0 27.211]);
ylabel('$2y/D_c$','interpreter','latex','fontsize',10,'position',[-26.576 0]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
set(h,'ticklabelinterpreter','latex','fontsize',10);
text(-17.44,-14.175,'$\mbox{(d)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.807 0.646 0.135 0.341]);
set(h,'position',[0.947 0.69 0.02 0.253]);

subplot(245);
plot(xout(1,:)/(Ds/2),S_thy(N/2+1,:));
hold on;
plot(xout(1,:)/(Ds/2),S_sim(N/2+1,:));
xlabel('$2x/D_c$','interpreter','latex','fontsize',10);
ylabel('$S^{\mbox{out}}\left(x,0\right)$','interpreter','latex','fontsize',10);
axis tight; grid on;
xlim([-1 1]*10); ylim([-0.01e-4 1.5e-4]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
text(-8.724,1.3575853658536586E-4,'$\mbox{(e)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.056+0.015 0.117 0.162 0.428]);
% set(gca,'yticklabel',[0 0.5 1 1.5]*1e-4);

subplot(246);
plot(yout(:,1)/(Ds/2),S_thy(:,N/2+1));
hold on;
plot(yout(:,1)/(Ds/2),S_sim(:,N/2+1));
xlabel('$2y/D_c$','interpreter','latex','fontsize',10);
ylabel('$S^{\mbox{out}}\left(0,y\right)$','interpreter','latex','fontsize',10);
% h = legend('$\mbox{Thy}$','$\mbox{Sim}$');
% set(h,'interpreter','latex','fontsize',10);
axis tight; grid on;
xlim([-1 1]*10); ylim([-0.01e-4 1.5e-4]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
text(-8.724,1.3575853658536586E-4,'$\mbox{(f)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.297+0.015 0.117 0.162 0.428]);

subplot(247);
plot(xout(1,:)/(Ds/2),mu_thy(N/2+1,:));
hold on;
plot(xout(1,:)/(Ds/2),mu_sim(N/2+1,:));
xlabel('$2x/D_c$','interpreter','latex','fontsize',10);
ylabel('$\left|\mu^{\mbox{out}}\left(x,0\right)\right|$','interpreter','latex','fontsize',10);
axis tight; grid on;
xlim([-1 1]*20); ylim([-0.01 1.01]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
text(-17.735,0.919,'$\mbox{(g)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.538+0.015 0.117 0.162 0.428]);

subplot(248);
plot(yout(:,1)/(Ds/2),mu_thy(:,N/2+1));
hold on;
plot(yout(:,1)/(Ds/2),mu_sim(:,N/2+1));
xlabel('$2y/D_c$','interpreter','latex','fontsize',10);
ylabel('$\left|\mu^{\mbox{out}}\left(0,y\right)\right|$','interpreter','latex','fontsize',10);
h = legend('$\mbox{Thy}$','$\mbox{Sim}$');
set(h,'interpreter','latex','fontsize',10);
set(h,'position',[0.428 0.506 0.104 0.095]);
axis tight; grid on;
xlim([-1 1]*20); ylim([-0.01 1.01]);
set(gca,'fontsize',10,'ticklabelinterpreter','latex');
text(-17.735,0.919,'$\mbox{(h)}$','interpreter','latex','fontsize',10,'backgroundcolor',[1 1 1]);
set(gca,'position',[0.792+0.015 0.117 0.162 0.428]);

